import java.io.*;
import java.util.*;

public class TestGen {
    int testCount, subtaskCount;
    PrintWriter testsDesc;
    Random random = new Random(49254308557L);
    public static final int MAX_N = 1000000;//10^6
    public static final int MAX_VAL = 1_000_000_000;

    private String folder;
    private String subtask;

    public int rand(int n) {
    	if (n < 1) return 0;
    	return random.nextInt(n);
    }

	public int rand() {
		return rand(MAX_N);
	}

    public void setFolder(String folder) {
        this.folder = folder;
    }

    public void setSubtask(String subtask) {
        this.subtask = subtask;
    }

    public void genSamples() throws IOException {
        setFolder("preliminary");
        setSubtask("subtask0");

        System.out.println("==== Generating samples ===");
        print(new Test(new int[]{3, 3, 1, 2, 2, 1}, "sample1"));
        print(new Test(new int[]{1, 1}, "simple2"));
        testCount = 0;
    }

    public void genHandleTests(int mx, int points) throws IOException {
    }

    public void genSubtask(int n, int points, int groupNo) throws IOException {
        ++subtaskCount;
        String subtask = "subtask" + subtaskCount;
        setSubtask(subtask);
        setFolder("tests");
        System.out.println("==== Generating group #" + groupNo + " ===");

        if (groupNo == 1) {
	        print(new Test(new int[]{1, 2}, "test1"));
    	    print(new Test(new int[]{3, 3, 3}, "test2"));
	        print(new Test(new int[]{1, 2}, "test4"));
    	    print(genOneToN(10, 4));
	        print(new Test(new int[]{1, 2, 2, 2, 1, 3, 3, 4, 4, 3}, "test6"));
	        print(new Test(new int[]{1, 2, 2, 1, 1, 4, 4, 4, 4, 4}, "test7"));
        }
        print(genOneToN(n, 4));
        print(genMaxAns(n - 2));
        print(genAllTheSame(n - 10, 239 + groupNo));
        for (int i = 0; i < 15; i++) {
			print(genMain(n - Integer.min(n - 10, rand(50)), rand(n)));
        }	
        testsDesc.println(String.format("comment_subtask%d=\"n &lt;= %d\"", subtaskCount, n));
        testsDesc.println(String.format("bonus_subtask%d=%d", subtaskCount, points));
    }


    private Test genMaxAns(int n) throws IOException {
    	int[] arr = new int[n];
    	int c1 = rand();
    	int c2 = rand();
		for (int i = 0; i < n; i++) {
			if (rand() % 2 == 1) {
				arr[i] = c1;
			} else {
				arr[i] = c2;
			}		
		}	
		return new Test(arr, String.format("max result n = %d", n));
	}
    
    private Test genAllTheSame(int n, int a) throws IOException {
    	int[] arr = new int[n];
		for (int i = 0; i < n; i++) {
			arr[i] = a;		
		}	
		return new Test(arr, String.format("all the same n = %d", n));
	}

    private Test genOneToN(int n, int a) throws IOException {
    	int[] arr = new int[n];
    	int st = rand(13456);
		for (int i = 0; i < n; i++) {
			arr[i] = st + i;		
		}	
		return new Test(arr, String.format("1 to n, n = %d", n));
	}
	
	private Test genMain(int n, int lenn) throws IOException {
	    int[] arr = new int[n];
	    int len = Integer.min(rand(lenn) + 1, n);
	    int st = rand(n - len);
   	 	for (int i = 0; i < st; i++) {
	    	int a = rand(MAX_VAL);
	    	arr[i] = a;
   	 	}
	    int c1 = rand(MAX_VAL);
	    int c2 = rand(MAX_VAL);
	    for (int i = st; i < Integer.min(st + len, n); i++) {
	    	int a = rand();
	    	if (a % 2 == 1) {
	    		arr[i] = c1;
    		} else {
    			arr[i] = c2;
	    	}
	    }
	    for (int i = st + len; i < n; i++) {
	    	int a = rand(MAX_VAL);
	    	arr[i] = a;
	    }	
	    return new Test(arr, String.format("random test, n = %d", n));
	}


    class Test {
        public String descr;
        public int n;
        public int[] arr;

        public Test(int[] arr, String descr) {
            this.n = arr.length;
            this.arr = arr;
            this.descr = descr;
        }
    }

    public void print(Test cur) throws IOException {
        String path = String.format("../%s/%02d", folder, ++testCount);
        PrintWriter out = new PrintWriter(path);

        System.out.println("Generating test #" + testCount + ": " + cur.descr);

        out.println(cur.n);
        for (int i = 0; i < cur.n; i++) {
        	out.print(cur.arr[i]);
        	if (i < cur.n - 1) {
        		out.print(" ");
        	}
        }
        out.println();
        out.close();
        testsDesc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=\"%s\"", path, folder, subtask, 0, cur.descr));
    }

    public void run() throws IOException {
        new File("../preliminary").mkdir();
        new File("../tests").mkdir();
        testsDesc = new PrintWriter("../tests.desc");

        //samples
        genSamples();
        //subtask1
        genSubtask(100, 30, 1);
        //subtask2
        genSubtask(1000, 30, 2);
        //subtask3
        genSubtask(MAX_N, 40, 3);

        testsDesc.close();
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}
